@@ -1,13 +1,9 @@
 package Blosxom::Header::Adapter;
 use strict;
 use warnings;
+use Carp qw/carp/;
 use CGI::Util qw/expires/;
 
-{
-    no strict 'refs';
-    *EXISTS = \&FETCH;
-}
-
 sub TIEHASH {
     my $class   = shift;
     my $adaptee = ref $_[0] eq 'HASH' ? shift : {};
@@ -26,6 +22,12 @@ sub TIEHASH {
         -p3p        => 'P3P',
     };
 
+    $self->{iterator} = {
+        collection => [],
+        current    => 0,
+        size       => 0,
+    };
+
     $self;
 }
 
@@ -63,10 +65,15 @@ sub FETCH {
         my $expires = $adaptee->{ $norm };
         return $expires ? expires( $expires ) : undef;
     }
+    elsif ( $norm eq '-date' and $self->has_date_header ) {
+        return expires( 0, 'http' );
+    }
 
     $adaptee->{ $norm };
 }
 
+*EXISTS = \&FETCH;
+
 sub STORE {
     my $self    = shift;
     my $norm    = $self->normalize( shift );
@@ -82,6 +89,10 @@ sub STORE {
     elsif ( $norm eq '-content_disposition' ) {
         delete $adaptee->{-attachment};
     }
+    elsif ( $norm eq '-date' and $self->has_date_header ) {
+        carp( 'The Date header is fixed' );
+        return;
+    }
 
     $adaptee->{ $norm } = $value;
 
@@ -104,6 +115,10 @@ sub DELETE {
         delete @{ $adaptee }{ $norm, '-attachment' };
         return $deleted;
     }
+    elsif ( $norm eq '-date' and $self->has_date_header ) {
+        carp( 'The Date header is fixed' );
+        return
+    }
 
     delete $adaptee->{ $norm };
 }
@@ -114,28 +129,37 @@ sub CLEAR {
 }
 
 sub FIRSTKEY {
-    my $self = shift;
-    keys %{ $self->{adaptee} };
-    exists $self->{adaptee}{-type} ? $self->NEXTKEY : 'Content-Type';
-};
+    my $self    = shift;
+    my $adaptee = $self->{adaptee};
 
-sub NEXTKEY {
-    my $self = shift;
-
-    my $nextkey;
-    while ( my ( $norm, $value ) = each %{ $self->{adaptee} } ) {
-        next if !$value or $norm eq '-charset' or $norm eq '-nph';
-        $nextkey = $self->denormalize( $norm );
-        last;
+    my @field_names;
+    push @field_names, 'Content-Type' unless exists $adaptee->{-type};
+    push @field_names, 'Date' if $self->has_date_header;
+    for my $norm ( keys %{ $adaptee } ) {
+        next unless $adaptee->{ $norm };
+        next if $norm eq '-charset' or $norm eq '-nph';
+        push @field_names, $self->denormalize( $norm );
     }
 
-    $nextkey;
+    %{ $self->{iterator} } = (
+        collection => \@field_names,
+        size       => scalar @field_names,
+        current    => 0,
+    );
+
+    $self->NEXTKEY;
+}
+
+sub NEXTKEY {
+    my $iterator = shift->{iterator};
+    return if $iterator->{current} >= $iterator->{size};
+    $iterator->{collection}->[ $iterator->{current}++ ];
 }
 
 sub SCALAR {
     my $self = shift;
     my $scalar = $self->FIRSTKEY;
-    keys %{ $self->{adaptee} } if $scalar;
+    $self->{iterator}->{current}-- if $scalar;
     $scalar;
 }
 
@@ -182,13 +206,18 @@ sub nph {
     $adaptee->{-nph};
 }
 
+sub has_date_header {
+    my $adaptee = shift->{adaptee};
+    $adaptee->{-expires} || $adaptee->{-cookie} || $adaptee->{-nph};
+}
+
 1;
 
 __END__
 
 =head1 NAME
 
-Blosxom::Header::Adapter
+Blosxom::Header::Adapter - Creates a case-insensitive hash
 
 =head1 SYNOPSIS
 
@@ -199,12 +228,68 @@ Blosxom::Header::Adapter
 
   tie my %adapter => 'Blosxom::Header::Adapter' => \%adaptee;
 
-  $adapter{Status} = '304 Not Modified';
+  $adapter{Content_Length} = 1234;
 
   print header( %adaptee );
+  # Content-length: 1234
+  # Content-Type: text/plain; charset=ISO-8859-1
+  #
 
 =head1 DESCRIPTION
 
+Creates a case-insensitive hash.
+
+=head2 BACKGROUND
+
+Blosxom, an weblog application, globalizes C<$header> which is a reference to
+a hash. This application passes C<$header> to C<CGI::header()> to generate HTTP
+headers.
+
+  package blosxom;
+  use strict;
+  use warnings;
+  use CGI qw/header/;
+
+  our $header = { -type => 'text/html' };
+
+  # Loads plugins
+
+  print header( $header );
+
+Plugins may modify C<$header> directly because the variable is global.
+On the other hand, C<header()> doesn't care whether C<keys> of C<$header> are
+lowercased nor start with a dash.
+There is no agreement with how to normalize C<keys> of C<$header>.
+
+=head2 HOW THIS MODULE NORMALIZES FIELD NAMES
+
+To specify field names consistently, we need to normalize them.
+If you follow one of normalization rules, you can modify C<$header>
+consistently. This module normalizes them as follows.
+
+Remember how Blosxom initializes C<$header>:
+
+  $header = { -type => 'text/html' };
+
+A key C<-type> is starting with a dash and lowercased, and so this module
+follows the same rules:
+
+  'Status'  # not normalized
+  'status'  # not normalized
+  '-status' # normalized
+
+How about C<Content-Length>? It contains a dash.
+To avoid quoting when specifying hash keys, this module transliterates dashes
+into underscores in field names:
+
+  'Content-Length'  # not normalized
+  '-content-length' # not normalized
+  '-content_length' # normalized
+
+If you follow the above normalization rule, you can modify C<$header> directly.
+In other words, this module is compatible with the way modifying C<$header>
+directly when you follow the above rule.
+
 =head2 METHODS
 
 =over 4
@@ -221,7 +306,9 @@ Associates a new hash instance with Blosxom::Header::Adapter.
 
 =item $bool = exists $adapter{ $field }
 
-=item $bool %adapter
+=item $bool = %adapter
+
+=item $field = each %adapter
 
 =item ( $field, $value ) = each %adapter
 
@@ -235,6 +322,20 @@ A shortcut for
 
 =item $adapter->attachment()
 
+=item $adapter->has_date_header()
+
+=back
+
+=head1 DIAGONOSTICS
+
+=over 4
+
+=item The Date header is fixed
+
+You attempted to modify the Date header when any of
+<-cookie>, C<-nph> or C<-expires> was set.
+See C<Blosxom::Header::date()>.
+
 =back
 
 =head1 SEE ALSO
